SystemOrganization addCategory: #'Zinc-WWS-Server'!

Object subclass: #ZnWebWorkspaceDelegate
	instanceVariableNames: 'workspaces endpoint'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WWS-Server'!

!ZnWebWorkspaceDelegate commentStamp: 'SvenVanCaekenberghe 1/28/2012 18:53' prior: 0!
ZnWebWorkspaceDelegate implements the http://ws.stfx.eu shared workspaces web service.

(ZnServer defaultOn: 1701)
	delegate: (ZnWebWorkspaceDelegate new endpoint: 'http://localhost:1701'; yourself);
	logToTranscript;
	start.
	
ZnEasy get: 'http://localhost:1701/1WS4U'
ZnEasy get: 'http://localhost:1701/1WS4U?format=text'
ZnEasy get: 'http://localhost:1701/1WS4U?format=html'

ZnEasy 
	post: 'http://localhost:1701' 
	data: (ZnEntity text: '123 factorial\3 timesRepeat: [ Smalltalk garbageCollect ]\' withCRs) 
!

!ZnWebWorkspaceDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/24/2012 23:20'!
demoKey
	^ '1WS4U'! !

!ZnWebWorkspaceDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 11:03'!
endpoint
	^ endpoint ifNil: [ endpoint := 'http://ws.stfx.eu' asZnUrl ]! !

!ZnWebWorkspaceDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/28/2012 18:37'!
endpoint: aUrlObject
	endpoint := aUrlObject asZnUrl ! !

!ZnWebWorkspaceDelegate methodsFor: 'private html generation' stamp: 'SvenVanCaekenberghe 2/2/2012 10:26'!
generateHomepage
	^ String streamContents: [ :stream |
		stream << ZnConstants standardHtmlDocType.
		stream << '<html>'.
		stream << self homepageData.
		stream << self standardPageFooter.
		stream << '</body></html>' ]
! !

!ZnWebWorkspaceDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/28/2012 13:06'!
generateNewKey
	^ ((10 raisedTo: 9) atRandom * Time primSecondsClock) printStringBase: 36
! !

!ZnWebWorkspaceDelegate methodsFor: 'private html generation' stamp: 'SvenVanCaekenberghe 1/31/2012 23:22'!
generatePageFor: workspace withKey: key
	^ String streamContents: [ :stream |
		stream << ZnConstants standardHtmlDocType; crlf.
		stream << '<html>' << (self standardPageHeader format: { key }).
		stream << '<body>'.
		stream << (self standardPageProlog format: { key }).
		stream << '<pre><code class="smalltalk">' << workspace << '</code></pre>'.
		stream << self standardPageFooter.
		stream << '</body></html>' ]
! !

!ZnWebWorkspaceDelegate methodsFor: 'private request handling' stamp: 'SvenVanCaekenberghe 1/28/2012 16:35'!
handleGet: request
	| key workspace format entity |
	key := request uri firstPathSegment.
	workspace := workspaces at: key ifAbsent: [ ^ ZnResponse notFound: request uri ].
	format := request uri queryAt: 'format' ifAbsent: [ nil ].
	entity := (format = 'html' or: [ (self isBrowserRequest: request) and: [ format ~= 'text' ] ])
		ifTrue: [ ZnEntity html: (self generatePageFor: workspace withKey: key) ]
		ifFalse: [ ZnEntity text: workspace ].
	^ ZnResponse ok: entity! !

!ZnWebWorkspaceDelegate methodsFor: 'private request handling' stamp: 'SvenVanCaekenberghe 1/28/2012 16:54'!
handleIndex: request
	^ ZnResponse ok: (ZnEntity html: self generateHomepage)! !

!ZnWebWorkspaceDelegate methodsFor: 'private request handling' stamp: 'SvenVanCaekenberghe 1/28/2012 13:06'!
handlePost: request
	| key |
	(request hasEntity and: [ request entity contentType isBinary not ])
		ifFalse: [ ^ ZnResponse badRequest: request ].
	key := self generateNewKey.
	workspaces at: key put: request contents.
	^ ZnResponse created: key! !

!ZnWebWorkspaceDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 3/7/2012 16:02'!
handleRequest: request
	request uri isSlash
		ifTrue: [
			request method = #GET ifTrue: [ ^ self handleIndex: request ].
			request method = #POST ifTrue: [ ^ self handlePost: request ] ]
		ifFalse: [
			(request uri pathSegments size = 1 and: [ request method = #GET ])
				ifTrue: [ ^ self handleGet: request ] ].
	^ ZnResponse badRequest: request uri! !

!ZnWebWorkspaceDelegate methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 2/2/2012 10:26'!
homepageData
	^ '
<head>
<title>Shared Smalltalk Workspaces</title>
<link href="http://stfx.eu/wws/style.css" rel="stylesheet" type="text/css"/>
</head>

<body>

<h1>Shared Smalltalk Workspaces</h1>
<h3>Hello and welcome to http://ws.stfx.eu,</h3>
<h3>a free web service to share Smalltalk workspaces.</h3>

<div class="intro">
<p>
This website offers a web service to easily <b>share Smalltalk workspaces</b>.
</p>
<p>
It is a <a href="http://en.wikipedia.org/wiki/Pastebin">pastebin</a> for Smalltalk.
</p>
<p>
Using a subclass of the standard Workspace <b>tool</b> called <b>ZnWorkspace</b>,
you can easily <b>publish the contents of any workspace to this website</b>.
</p>
<p>
Each time you publish a workspace, you will create a new shared web workspace.
Each shared web workspace is identified by a unique key.
Keys look like 6DKGPHMWVMI8. The <b>unique, official URL</b> to the shared workspace
for that key would be http://ws.stfx.eu/6DKGPHMWVMI8.
</p>
<p>
To actually <b>share the workspace</b> you can <b>pass the URL around</b> in any message.
Recepients will then be able to <b>open the shared workspace using a web browser</b>.
The HTML page generated will apply Smalltalk aware syntax highlighting to the contents.
</p>
<p>
Alternatively, recipients of a shared workspace URL or key can <b>use ZnWorkspace
to open the contents right in an image</b>, ready to be used.
</p>
<p>
For instructions on how to begin, see the <a href="http://ws.stfx.eu/1WS4U">Getting Started Workspace</a>.
</p>
<p>
All information published via this service is public.
The keys in the URLs are random and hard to guess, 
so unless someone knows the URL/key they won''t be able to access the workspace.
We promise not to further share workspaces by publishing indexes or making their contents searcheable.
This service is currently in beta.
Workspaces are not guaranteed to be persistent,
although every effort is made to offer a good service.
</p>
<p>
When serving a workspace''s contents, the Accept HTTP header will normally determine the resulting response''s content.
Browser HTTP agents that normally indicate that they prefer HTML content will receive a full, decorated HTML page.
Non-browser HTTP agents will receive plain, undecorated text content. 
A custom Accept header can be used to control the response.
Additionally, adding a query parameter format=html or format=text to the URL will override this behavior. 
</p>
<p>
Once created, shared workspaces are read-only. They can neither be modified, nor deleted.
Little used workspaces will eventually be removed after some period of time.
The maximum size for workspaces is 64Kb. 
Any line end convention as well as ASCII, Latin1 and UTF-8 encoding are supported for incoming data.
Workspaces are always served using the Unix line end convention and UTF-8 encoding.
</p>
</div>
'! !

!ZnWebWorkspaceDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/28/2012 16:52'!
initialize 
	super initialize.
	workspaces := ZnWebWorkspaceStorage on: (FileDirectory default / 'wws' / 'data')! !

!ZnWebWorkspaceDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/28/2012 16:56'!
isBrowserRequest: request
	| accept primaryMimeType |
	accept := request headers at: 'Accept' ifAbsent: [ ^ false ].
	primaryMimeType := (((accept findTokens: ',') 
		collect: [ :each | each asZnMimeType])
			sorted: [ :x :y | | q1 q2 |
				q1 := (x parameterAt: 'q' ifAbsent: [ 1.0 ]) asNumber.
				q2 := (y parameterAt: 'q' ifAbsent: [ 1.0 ]) asNumber.
				q1 >= q2 ]) first.
	^ primaryMimeType = ZnMimeType textHtml ! !

!ZnWebWorkspaceDelegate methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 1/31/2012 22:19'!
standardPageFooter
	^ '<div class="footer">
<a href="http://ws.stfx.eu">Shared Smalltalk Workspaces</a>
 is a free web service offered by
<a href="http://stfx.eu">http://stfx.eu</a>. 
Copyright &copy; 2012 Sven Van Caekenberghe.
Built and served using <a href="http://www.pharo.st">Pharo Smalltalk</a> 1.4.
</div>
<div class="disclaimer">
THIS SOFTWARE SERVICE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHOR OR SERVICE PROVIDER BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE SERVICE OR THE USE OR OTHER DEALINGS 
IN THE SOFTWARE SERVICE.
</div>'! !

!ZnWebWorkspaceDelegate methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 1/31/2012 23:18'!
standardPageHeader
	^ '<head>
<title>Shared Smalltalk Workspace {1} - http://ws.stfx.eu/{1}</title>
<link href="http://stfx.eu/wws/style.css" rel="stylesheet" type="text/css"/>
<script src="http://stfx.eu/wws/highlight/highlight.pack.js" type="text/javascript"></script>
<script type="text/javascript">
  hljs.initHighlightingOnLoad();
</script>
<link href="http://stfx.eu/wws/highlight/styles/default.css" rel="stylesheet"" type="text/css"/>
</head>
'! !

!ZnWebWorkspaceDelegate methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 2/2/2012 22:25'!
standardPageProlog
	^ '<h1>Shared Smalltalk Workspace {1}<h1>
<h3><a href="{1}">http://ws.stfx.eu/{1}</a></h3>
<div class="toolbar">
Doit <input class="doit" type="text" value="ZnWorkspace openUrl: ''http://ws.stfx.eu/{1}''." size="60" readonly="true"/>
| <a href="http://ws.stfx.eu/{1}?format=text">Raw</a>
| For instructions, see the <a href="http://ws.stfx.eu/1WS4U">Getting Started Workspace</a>
| <a href="http://ws.stfx.eu">Home</a>
</div>
'! !

Object subclass: #ZnWebWorkspaceStorage
	instanceVariableNames: 'directory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-WWS-Server'!

!ZnWebWorkspaceStorage commentStamp: 'SvenVanCaekenberghe 1/25/2012 16:31' prior: 0!
ZnWebWorkspaceStorage stores a dictionary of keys with string contents in a directory.

To allow for a large number of key/values subdirectories are created based on the first two letters of a key.!

!ZnWebWorkspaceStorage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/25/2012 16:27'!
on: directory
	^ self new
		directory: directory;
		yourself! !

!ZnWebWorkspaceStorage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2012 16:29'!
at: key
	^ self at: key ifAbsent: [ KeyNotFound signalFor: key ]! !

!ZnWebWorkspaceStorage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2012 22:09'!
at: key ifAbsent: block
	| fullName entry |
	fullName := self directory / (key first: 2) / key.
	(fullName containingDirectory fileExists: fullName localName)  
		ifFalse: [ ^ block value ].
	entry := fullName directoryEntry.
	^ FileStream 
		fileNamed: entry fullName 
		do: [ :stream | stream upToEnd ]! !

!ZnWebWorkspaceStorage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/29/2012 20:30'!
at: key put: string
	| subDirectory |
	self directory assureExistence.
	subDirectory := self directory / (key first: 2).
	subDirectory assureExistence.
	FileStream 
		fileNamed: (subDirectory / key) pathName  
		do: [ :stream | 
			stream lineEndConvention: #lf.
			string linesDo: [ :each | stream nextPutAll: each; cr ] ]
! !

!ZnWebWorkspaceStorage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2012 16:27'!
directory
	^ directory! !

!ZnWebWorkspaceStorage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/25/2012 16:28'!
directory: aDirectory
	directory := aDirectory! !
